@@ -58,7 +58,6 @@
 #include "nsIScrollableFrame.h"
 #include "nsINameSpaceManager.h"
 #include "nsINodeInfo.h"
-#include "nsIPrefService.h"
 #include "nsRootAccessible.h"
 #include "nsIServiceManager.h"
 #include "nsTextFormatter.h"
@@ -485,18 +484,22 @@ STDMETHODIMP nsAccessibleWrap::get_accKeyboardShortcut(
       /* [retval][out] */ BSTR __RPC_FAR *pszKeyboardShortcut)
 {
 __try {
+  if (!pszKeyboardShortcut)
+    return E_INVALIDARG;
+
   *pszKeyboardShortcut = NULL;
   nsAccessible *xpAccessible = GetXPAccessibleFor(varChild);
-  if (xpAccessible) {
-    nsAutoString shortcut;
-    nsresult rv = xpAccessible->GetKeyboardShortcut(shortcut);
-    if (NS_FAILED(rv))
-      return E_FAIL;
+  if (!xpAccessible || xpAccessible->IsDefunct())
+    return E_FAIL;
 
-    *pszKeyboardShortcut = ::SysAllocStringLen(shortcut.get(),
-                                               shortcut.Length());
-    return *pszKeyboardShortcut ? S_OK : E_OUTOFMEMORY;
-  }
+  nsAutoString shortcut;
+  nsresult rv = xpAccessible->GetKeyboardShortcut(shortcut);
+  if (NS_FAILED(rv))
+    return GetHRESULT(rv);
+
+  *pszKeyboardShortcut = ::SysAllocStringLen(shortcut.get(),
+                                             shortcut.Length());
+  return *pszKeyboardShortcut ? S_OK : E_OUTOFMEMORY;
 } __except(FilterA11yExceptions(::GetExceptionCode(), GetExceptionInformation())) { }
   return E_FAIL;
 }
@@ -900,38 +903,20 @@ STDMETHODIMP nsAccessibleWrap::accHitTest(
 {
 __try {
   VariantInit(pvarChild);
+  if (IsDefunct())
+    return E_FAIL;
 
-  // convert to window coords
-  nsCOMPtr<nsIAccessible> xpAccessible;
-
-  xLeft = xLeft;
-  yTop = yTop;
-
-  if (nsAccUtils::MustPrune(this)) {
-    xpAccessible = this;
-  }
-  else {
-    GetChildAtPoint(xLeft, yTop, getter_AddRefs(xpAccessible));
-  }
+  nsAccessible* accessible = ChildAtPoint(xLeft, yTop, eDirectChild);
 
   // if we got a child
-  if (xpAccessible) {
+  if (accessible) {
     // if the child is us
-    if (xpAccessible == static_cast<nsIAccessible*>(this)) {
+    if (accessible == this) {
       pvarChild->vt = VT_I4;
       pvarChild->lVal = CHILDID_SELF;
     } else { // its not create an Accessible for it.
       pvarChild->vt = VT_DISPATCH;
-      pvarChild->pdispVal = NativeAccessible(xpAccessible);
-      nsCOMPtr<nsIAccessNode> accessNode(do_QueryInterface(xpAccessible));
-      NS_ASSERTION(accessNode, "Unable to QI to nsIAccessNode");
-      nsCOMPtr<nsIDOMNode> domNode;
-      accessNode->GetDOMNode(getter_AddRefs(domNode));
-      if (!domNode) {
-        // Has already been shut down
-        pvarChild->vt = VT_EMPTY;
-        return E_FAIL;
-      }
+      pvarChild->pdispVal = NativeAccessible(accessible);
     }
   } else {
     // no child at that point
@@ -1383,19 +1368,18 @@ STDMETHODIMP
 nsAccessibleWrap::get_indexInParent(long *aIndexInParent)
 {
 __try {
+  if (!aIndexInParent)
+    return E_INVALIDARG;
+
   *aIndexInParent = -1;
+  if (IsDefunct())
+    return E_FAIL;
 
-  PRInt32 index = -1;
-  nsresult rv = GetIndexInParent(&index);
-  if (NS_FAILED(rv))
-    return GetHRESULT(rv);
-
-  if (index == -1)
+  *aIndexInParent = IndexInParent();
+  if (*aIndexInParent == -1)
     return S_FALSE;
 
-  *aIndexInParent = index;
   return S_OK;
-
 } __except(nsAccessNodeWrap::FilterA11yExceptions(::GetExceptionCode(), GetExceptionInformation())) { }
   return E_FAIL;
 }
