@@ -334,7 +334,11 @@ struct AutoCXPusher
   }
 };
 
-static JSContext *
+namespace mozilla {
+namespace plugins {
+namespace parent {
+
+JSContext *
 GetJSContext(NPP npp)
 {
   NS_ENSURE_TRUE(npp, nsnull);
@@ -360,6 +364,9 @@ GetJSContext(NPP npp)
   return (JSContext *)scx->GetNativeContext();
 }
 
+}
+}
+}
 
 static NPP
 LookupNPP(NPObject *npobj);
