@@ -58,8 +58,6 @@
 #include "nsEditorUtils.h"
 #include "EditTxn.h"
 #include "nsEditProperty.h"
-#include "nsIPrefBranch.h"
-#include "nsIPrefService.h"
 #include "nsUnicharUtils.h"
 #include "nsILookAndFeel.h"
 #include "nsWidgetsCID.h"
@@ -70,6 +68,10 @@
 // for IBMBIDI
 #include "nsFrameSelection.h"
 
+#include "mozilla/Preferences.h"
+
+using namespace mozilla;
+
 static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
 
 #define CANCEL_OPERATION_IF_READONLY_OR_DISABLED \
@@ -157,13 +159,8 @@ nsTextEditRules::Init(nsPlaintextEditor *aEditor)
     NS_ENSURE_SUCCESS(res, res);
   }
 
-  PRBool deleteBidiImmediately = PR_FALSE;
-  nsCOMPtr<nsIPrefBranch> prefBranch =
-    do_GetService(NS_PREFSERVICE_CONTRACTID, &res);
-  if (NS_SUCCEEDED(res))
-    prefBranch->GetBoolPref("bidi.edit.delete_immediately",
-                            &deleteBidiImmediately);
-  mDeleteBidiImmediately = deleteBidiImmediately;
+  mDeleteBidiImmediately =
+    Preferences::GetBool("bidi.edit.delete_immediately", PR_FALSE);
 
   return res;
 }
@@ -234,22 +231,6 @@ nsTextEditRules::AfterEdit(PRInt32 action, nsIEditor::EDirection aDirection)
 
     // collapse the selection to the trailing BR if it's at the end of our text node
     CollapseSelectionToTrailingBRIfNeeded(selection);
-    
-    /* After inserting text the cursor Bidi level must be set to the level of the inserted text.
-     * This is difficult, because we cannot know what the level is until after the Bidi algorithm
-     * is applied to the whole paragraph.
-     *
-     * So we set the cursor Bidi level to UNDEFINED here, and the caret code will set it correctly later
-     */
-    if (action == nsEditor::kOpInsertText
-        || action == nsEditor::kOpInsertIMEText) {
-      nsCOMPtr<nsISelectionPrivate> privateSelection(do_QueryInterface(selection));
-      nsRefPtr<nsFrameSelection> frameSelection;
-      privateSelection->GetFrameSelection(getter_AddRefs(frameSelection));      
-      if (frameSelection) {
-        frameSelection->UndefineCaretBidiLevel();
-      }
-    }
   }
   return res;
 }
