@@ -59,6 +59,8 @@ import android.hardware.*;
 
 import android.util.*;
 import android.net.*;
+import android.database.*;
+import android.provider.*;
 
 abstract public class GeckoApp
     extends Activity
@@ -137,7 +139,7 @@ abstract public class GeckoApp
                         mLibLoadThread.join();
                 } catch (InterruptedException ie) {}
                 surfaceView.mSplashStatusMsg =
-                    getResources().getString(R.string.splash_screen_label);
+                    getResources().getString(R.string.splash_screen_loading);
                 surfaceView.drawSplashScreen();
                 // unpack files in the components directory
                 try {
@@ -161,10 +163,17 @@ abstract public class GeckoApp
                 }
 
                 // and then fire us up
-                String env = i.getStringExtra("env0");
-                GeckoAppShell.runGecko(getApplication().getPackageResourcePath(),
-                                       i.getStringExtra("args"),
-                                       i.getDataString());
+                try {
+                    String env = i.getStringExtra("env0");
+                    GeckoAppShell.runGecko(getApplication().getPackageResourcePath(),
+                                           i.getStringExtra("args"),
+                                           i.getDataString());
+                } catch (Exception e) {
+                    Log.e("GeckoApp", "top level exception", e);
+                    StringWriter sw = new StringWriter();
+                    e.printStackTrace(new PrintWriter(sw));
+                    GeckoAppShell.reportJavaCrash(sw.toString());
+                }
             }
         }.start();
         return true;
@@ -177,6 +186,19 @@ abstract public class GeckoApp
         mAppContext = this;
         mMainHandler = new Handler();
 
+        mMainHandler.post(new Runnable() {
+            public void run() {
+                try {
+                    Looper.loop();
+                } catch (Exception e) {
+                    Log.e("GeckoApp", "top level exception", e);
+                    StringWriter sw = new StringWriter();
+                    e.printStackTrace(new PrintWriter(sw));
+                    GeckoAppShell.reportJavaCrash(sw.toString());
+                }
+            }
+        });
+
         SharedPreferences settings = getPreferences(Activity.MODE_PRIVATE);
         String localeCode = settings.getString(getPackageName() + ".locale", "");
         if (localeCode != null && localeCode.length() > 0)
@@ -238,10 +260,10 @@ abstract public class GeckoApp
              new File(getApplication().getPackageResourcePath()).lastModified()
              >= libxulFile.lastModified()))
             surfaceView.mSplashStatusMsg =
-                getResources().getString(R.string.splash_screen_installing);
+                getResources().getString(R.string.splash_screen_installing_libs);
         else
             surfaceView.mSplashStatusMsg =
-                getResources().getString(R.string.splash_screen_label);
+                getResources().getString(R.string.splash_screen_loading);
         mLibLoadThread.start();
     }
 
@@ -559,7 +581,7 @@ abstract public class GeckoApp
         Log.i("GeckoAppJava", "Update is available!");
 
         // Launch APK
-        File updateFileToRun = new File(updateDir + getPackageName() + "-update.apk");
+        File updateFileToRun = new File(updateDir, getPackageName() + "-update.apk");
         try {
             if (updateFile.renameTo(updateFileToRun)) {
                 String amCmd = "/system/bin/am start -a android.intent.action.VIEW " +
@@ -634,13 +656,33 @@ abstract public class GeckoApp
             try {
                 ContentResolver cr = getContentResolver();
                 Uri uri = data.getData();
-                String mimeType = cr.getType(uri);
-                String fileExt = "." +
-                    GeckoAppShell.getExtensionFromMimeType(mimeType);
-                File file =
-                    File.createTempFile("tmp_" +
-                                        (int)Math.floor(1000 * Math.random()),
-                                        fileExt, sGREDir);
+                Cursor cursor = GeckoApp.mAppContext.getContentResolver().query(
+                    uri, 
+                    new String[] { OpenableColumns.DISPLAY_NAME },
+                    null, 
+                    null, 
+                    null);
+                String name = null;
+                if (cursor != null) {
+                    try {
+                        if (cursor.moveToNext()) {
+                            name = cursor.getString(0);
+                        }
+                    } finally {
+                        cursor.close();
+                    }
+                }
+                String fileName = "tmp_";
+                String fileExt = null;
+                int period;
+                if (name == null || (period = name.lastIndexOf('.')) == -1) {
+                    String mimeType = cr.getType(uri);
+                    fileExt = "." + GeckoAppShell.getExtensionFromMimeType(mimeType);
+                } else {
+                    fileExt = name.substring(period);
+                    fileName = name.substring(0, period);
+                }
+                File file = File.createTempFile(fileName, fileExt, sGREDir);
 
                 FileOutputStream fos = new FileOutputStream(file);
                 InputStream is = cr.openInputStream(uri);
