@@ -1,5 +1,4 @@
-/* -*- Mode: java; tab-width:8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -16,11 +15,11 @@
  * The Original Code is JavaScript Engine testing utilities.
  *
  * The Initial Developer of the Original Code is
- * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Jeff Walden <jwalden+code@mit.edu>.
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,27 +36,35 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-var summary = 'GetXMLEntity should not assume FastAppendChar is infallible';
-var BUGNUMBER = 458679;
-var actual = '';
-var expect = '';
+//-----------------------------------------------------------------------------
+var BUGNUMBER     = "350206";
+var summary = "Order of destructuring, destructuring in the presence of " +
+              "exceptions";
+var actual, expect;
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
+/**************
+ * BEGIN TEST *
+ **************/
+
+var failed = "No crash";
+
 try
 {
-    var x = "<";
+  var t1 = <ns2:t1 xmlns:ns2="http://bar/ns2"/>;
+  var n2 = new Namespace("http://ns2");
+  t1.@n2::a1 = "a1 from ns2";
 
-    while (x.length < 12000000)
-        x += x;
-
-    <e4x>{x}</e4x>;
+  t1.toXMLString();
 }
-catch(ex)
+catch (ex)
 {
+  failed = ex;
 }
 
+expect = "No crash";
+actual = failed;
+
 TEST(1, expect, actual);
-
-END();
