@@ -38,25 +38,28 @@
 
 
 /**
-   File Name:          15.5.4.1.js
-   ECMA Section:       15.5.4.1 String.prototype.constructor
+   File Name:          15.5.3.1-3.js
+   ECMA Section:       15.5.3.1 Properties of the String Constructor
+
+   Description:        The initial value of String.prototype is the built-in
+   String prototype object.
+
+   This property shall have the attributes [ DontEnum,
+   DontDelete, ReadOnly]
+
+   This tests the DontDelete attribute.
 
-   Description:
    Author:             christine@netscape.com
-   Date:               28 october 1997
-
+   Date:               1 october 1997
 */
-var SECTION = "15.5.4.1";
+
+var SECTION = "15.5.3.1-3";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "String.prototype.constructor";
+var TITLE   = "Properties of the String Constructor";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase( SECTION, "String.prototype.constructor == String",  true, String.prototype.constructor == String );
-
-new TestCase( SECTION, "var STRING = new String.prototype.constructor('hi'); STRING.getClass = Object.prototype.toString; STRING.getClass()",
-	      "[object String]",
-	      eval("var STRING = new String.prototype.constructor('hi'); STRING.getClass = Object.prototype.toString; STRING.getClass()") );
+new TestCase( SECTION,	"delete( String.prototype )",   false,   eval("delete ( String.prototype )") );
 
 test();
