@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Mathieu Fenniak
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,12 +36,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 351070;
-var summary = 'decompilation of let declaration should not change scope';
-var summarytrunk = 'let declaration must be direct child of block or top-level implicit block';
-
-var actual = '';
-var expect = '';
+var BUGNUMBER = 457093;
+var summary = 'Do not assert: newtop <= oldtop, decompiling function';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 
 //-----------------------------------------------------------------------------
@@ -53,49 +51,26 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  var f;
-  var c;
-
-  try
+ 
+  function Test(object)
   {
-    c = '(function () { var a = 2; if (!!true) let a = 3; return a; })';
-    f = eval(c);
-    expect = 'function () { var a = 2; if (!!true) let a = 3; return a; }';
-    actual = f + '';
-    compareSource(expect, actual, summary);
-
-    expect = 3;
-    actual = f();
-    reportCompare(expect, actual, summary);
-  }
-  catch(ex)
-  {
-    // See https://bugzilla.mozilla.org/show_bug.cgi?id=408957
-    expect = 'SyntaxError';
-    actual = ex.name;
-    reportCompare(expect, actual, summarytrunk + ': ' + c);
+    for (var i in object)
+    {
+      try
+      {
+      }
+      catch(E) {}
+    }
   }
 
-  try
-  {
-    c = '(function () { var a = 2; if (!!true) {let a = 3;} return a; })';
-    f = eval(c);
-    expect = 'function () { var a = 2; if (!!true) { let a = 3;} return a; }';
-    actual = f + '';
-    compareSource(expect, actual, summary);
+  var x = Test.toString();
+  
+  print(x);
 
-    expect = 2;
-    actual = f();
-    reportCompare(expect, actual, summary);
-  }
-  catch(ex)
-  {
-    // See https://bugzilla.mozilla.org/show_bug.cgi?id=408957
-    expect = 'SyntaxError';
-    actual = ex.name;
-    reportCompare(expect, actual, summarytrunk + ': ' + c);
-  }
+  expect = 'function Test ( object ) { for ( var i in object ) { try { } catch ( E ) { } } }';
+  actual = Test + '';
+
+  compareSource(expect, actual, summary);
 
   exitFunc ('test');
 }
