@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,11 +36,12 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 354499;
-var summary = 'Iterating over Array elements';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 455464;
+var summary = 'Do not assert with JIT: !TRACE_RECORDER(cx) ^ (jumpTable == recordingJumpTable)';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
+a=b=c=d=0; this.__defineGetter__('g', gc); for each (y in this);
 
 //-----------------------------------------------------------------------------
 test();
@@ -52,25 +53,11 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  expect = actual = 'No Crash';
+  jit(true);
 
-  var obj = {get a(){ return new Object(); }};
+  a=b=c=d=0; this.__defineGetter__('g', gc); for each (y in this);
 
-  function setter(v)
-  {
-    // Push out obj.a from all temp roots
-    var tmp = { get toString() { return new Object(); }};
-    try { String(tmp); } catch (e) {  }
-    gc();
-  }
-
-  Array.prototype.__defineGetter__(0, function() { });
-  Array.prototype.__defineSetter__(0, setter);
-
-  for (var i in Iterator(obj))
-    print(uneval(i));
-
-  delete Array.prototype[0];
+  jit(false);
 
   reportCompare(expect, actual, summary);
 
