@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Brendan Eich
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,10 +36,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 385133;
-var summary = 'Do not crash due to recursion with watch, setter, delete, generator';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
+var actual = '';
+var expect = '';
 
 
 //-----------------------------------------------------------------------------
@@ -52,16 +52,15 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  try
-  { 
-    Object.defineProperty(this, "x", { set: {}.watch, enumerable: true, configurable: true });
-    function g() { x = 1; yield; }
-    g().next();
-  }
-  catch(ex)
-  {
-    print(ex + '');
+// ------- Comment #63 From Brendan Eich
+
+  function f(that) {
+    for (ix in this)
+      print(ix);
+    for (let ix in that)
+      ;
   }
+
   reportCompare(expect, actual, summary);
 
   exitFunc ('test');
