@@ -57,7 +57,6 @@
 #include "nsIURI.h"
 #include "nsJSEnvironment.h"
 #include "plbase64.h"
-#include "jstypedarray.h"
 
 #include "XrayWrapper.h"
 #include "WrapperFactory.h"
@@ -398,8 +397,6 @@ nsXPConnect::Collect()
     if(!ccx.IsValid())
         return;
 
-    nsXPConnect::GetRuntimeInstance()->ClearWeakRoots();
-
     JSContext *cx = ccx.GetJSContext();
 
     // We want to scan the current thread for GC roots only if it was in a
@@ -426,7 +423,11 @@ nsXPConnect::GarbageCollect()
 
 // JSTRACE_XML can recursively hold on to more JSTRACE_XML objects, adding it to
 // the cycle collector avoids stack overflow.
-#define ADD_TO_CC(_kind)    ((_kind) == JSTRACE_OBJECT || (_kind) == JSTRACE_XML)
+inline bool
+AddToCCKind(uint32 kind)
+{
+    return kind == JSTRACE_OBJECT || kind == JSTRACE_XML;
+}
 
 #ifdef DEBUG_CC
 struct NoteJSRootTracer : public JSTracer
@@ -444,7 +445,7 @@ struct NoteJSRootTracer : public JSTracer
 static void
 NoteJSRoot(JSTracer *trc, void *thing, uint32 kind)
 {
-    if(ADD_TO_CC(kind))
+    if(AddToCCKind(kind))
     {
         NoteJSRootTracer *tracer = static_cast<NoteJSRootTracer*>(trc);
         PLDHashEntryHdr *entry = PL_DHashTableOperate(tracer->mObjects, thing,
@@ -542,7 +543,7 @@ nsXPConnect::FinishCycleCollection()
 nsCycleCollectionParticipant *
 nsXPConnect::ToParticipant(void *p)
 {
-    if (!ADD_TO_CC(js_GetGCThingTraceKind(p)))
+    if (!AddToCCKind(js_GetGCThingTraceKind(p)))
         return NULL;
     return this;
 }
@@ -581,8 +582,8 @@ nsXPConnect::Unroot(void *p)
 JSBool
 xpc_GCThingIsGrayCCThing(void *thing)
 {
-    uint32 kind = js_GetGCThingTraceKind(thing);
-    return ADD_TO_CC(kind) && xpc_IsGrayGCThing(thing);
+    return AddToCCKind(js_GetGCThingTraceKind(thing)) &&
+           xpc_IsGrayGCThing(thing);
 }
 
 /*
@@ -615,7 +616,7 @@ UnmarkGrayChildren(JSTracer *trc, void *thing, uint32 kind)
     }
 
     // If this thing is not a CC-kind or already non-gray then we're done.
-    if(!ADD_TO_CC(kind) || !xpc_IsGrayGCThing(thing))
+    if(!AddToCCKind(kind) || !xpc_IsGrayGCThing(thing))
         return;
 
     // Unmark.
@@ -659,7 +660,7 @@ struct TraversalTracer : public JSTracer
 static void
 NoteJSChild(JSTracer *trc, void *thing, uint32 kind)
 {
-    if(ADD_TO_CC(kind))
+    if(AddToCCKind(kind))
     {
         TraversalTracer *tracer = static_cast<TraversalTracer*>(trc);
 
@@ -745,7 +746,7 @@ nsXPConnect::Traverse(void *p, nsCycleCollectionTraversalCallback &cb)
         }
     }
 
-    CCNodeType type;
+    PRBool isMarked;
 
 #ifdef DEBUG_CC
     // Note that the conditions under which we specify GCMarked vs.
@@ -763,14 +764,13 @@ nsXPConnect::Traverse(void *p, nsCycleCollectionTraversalCallback &cb)
         // ExplainLiveExpectedGarbage codepath
         PLDHashEntryHdr* entry =
             PL_DHashTableOperate(&mJSRoots, p, PL_DHASH_LOOKUP);
-        type = markJSObject || PL_DHASH_ENTRY_IS_BUSY(entry) ? GCMarked :
-                                                               GCUnmarked;
+        isMarked = markJSObject || PL_DHASH_ENTRY_IS_BUSY(entry);
     }
     else
 #endif
     {
         // Normal codepath (matches non-DEBUG_CC codepath).
-        type = !markJSObject && xpc_IsGrayGCThing(p) ? GCUnmarked : GCMarked;
+        isMarked = markJSObject || !xpc_IsGrayGCThing(p);
     }
 
     if (cb.WantDebugInfo()) {
@@ -843,19 +843,19 @@ nsXPConnect::Traverse(void *p, nsCycleCollectionTraversalCallback &cb)
             char fullname[100];
             JS_snprintf(fullname, sizeof(fullname),
                         "%s (global=%p)", name, global);
-            cb.DescribeNode(type, 0, sizeof(JSObject), fullname);
+            cb.DescribeGCedNode(isMarked, sizeof(JSObject), fullname);
         } else {
-            cb.DescribeNode(type, 0, sizeof(JSObject), name);
+            cb.DescribeGCedNode(isMarked, sizeof(JSObject), name);
         }
     } else {
-        cb.DescribeNode(type, 0, sizeof(JSObject), "JS Object");
+        cb.DescribeGCedNode(isMarked, sizeof(JSObject), "JS Object");
     }
 
     // There's no need to trace objects that have already been marked by the JS
     // GC. Any JS objects hanging from them will already be marked. Only do this
     // if DEBUG_CC is not defined, else we do want to know about all JS objects
     // to get better graphs and explanations.
-    if(!cb.WantAllTraces() && type == GCMarked)
+    if(!cb.WantAllTraces() && isMarked)
         return NS_OK;
 
     TraversalTracer trc(cb);
@@ -930,8 +930,7 @@ public:
         // edges will ensure that any cycles this context is in won't be
         // collected.
         unsigned refCount = nsXPConnect::GetXPConnect()->GetOutstandingRequests(cx) + 1;
-
-        cb.DescribeNode(RefCounted, refCount, sizeof(JSContext), "JSContext");
+        NS_IMPL_CYCLE_COLLECTION_DESCRIBE(JSContext, refCount)
         NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "[global object]");
         if (cx->globalObject) {
             cb.NoteScriptChild(nsIProgrammingLanguage::JAVASCRIPT,
@@ -969,23 +968,6 @@ inline nsresult UnexpectedFailure(nsresult rv)
     return rv;
 }
 
-class SaveFrame
-{
-public:
-    SaveFrame(JSContext *cx)
-        : mJSContext(cx) {
-        mFrame = JS_SaveFrameChain(mJSContext);
-    }
-
-    ~SaveFrame() {
-        JS_RestoreFrameChain(mJSContext, mFrame);
-    }
-
-private:
-    JSContext *mJSContext;
-    JSStackFrame *mFrame;
-};
-
 /* void initClasses (in JSContextPtr aJSContext, in JSObjectPtr aGlobalJSObj); */
 NS_IMETHODIMP
 nsXPConnect::InitClasses(JSContext * aJSContext, JSObject * aGlobalJSObj)
@@ -2194,7 +2176,6 @@ nsXPConnect::ReleaseJSContext(JSContext * aJSContext, PRBool noGC)
                    (void *)aJSContext);
 #endif
             ccx->SetDestroyJSContextInDestructor(JS_TRUE);
-            JS_ClearNewbornRoots(aJSContext);
             return NS_OK;
         }
         // else continue on and synchronously destroy the JSContext ...
