@@ -84,9 +84,6 @@ NS_NewLayoutDebugger(nsILayoutDebugger** aResult)
     return NS_ERROR_NULL_POINTER;
   }
   nsLayoutDebugger* it = new nsLayoutDebugger();
-  if (!it) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
   return it->QueryInterface(NS_GET_IID(nsILayoutDebugger), (void**)aResult);
 }
 
@@ -213,7 +210,7 @@ void
 nsFrame::PrintDisplayList(nsDisplayListBuilder* aBuilder,
                           const nsDisplayList& aList)
 {
-  PrintDisplayListTo(aBuilder, aList, 0, stderr);
+  PrintDisplayListTo(aBuilder, aList, 0, stdout);
 }
 
 #endif
