@@ -54,7 +54,7 @@
 #include "nsIChannelEventSink.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsContentUtils.h"
-#include "nsIPrefService.h"
+#include "mozilla/Preferences.h"
 
 #include "nsPresContext.h"
 #include "nsIPresShell.h"
@@ -75,6 +75,8 @@
 
 #include "nsStyleSet.h"
 
+using namespace mozilla;
+
 #ifdef PR_LOGGING
 static PRLogModuleInfo *gFontDownloaderLog = PR_NewLogModule("fontdownloader");
 #endif /* PR_LOGGING */
@@ -88,6 +90,7 @@ nsFontFaceLoader::nsFontFaceLoader(gfxProxyFontEntry *aProxy, nsIURI *aFontURI,
   : mFontEntry(aProxy), mFontURI(aFontURI), mFontSet(aFontSet),
     mChannel(aChannel)
 {
+  mFontFamily = aProxy->Family();
 }
 
 nsFontFaceLoader::~nsFontFaceLoader()
@@ -104,11 +107,8 @@ nsFontFaceLoader::~nsFontFaceLoader()
 void
 nsFontFaceLoader::StartedLoading(nsIStreamLoader *aStreamLoader)
 {
-  PRInt32 loadTimeout = 3000;
-  nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (prefs) {
-    prefs->GetIntPref("gfx.downloadable_fonts.fallback_delay", &loadTimeout);
-  }
+  PRInt32 loadTimeout =
+    Preferences::GetInt("gfx.downloadable_fonts.fallback_delay", 3000);
   if (loadTimeout > 0) {
     mLoadTimer = do_CreateInstance("@mozilla.org/timer;1");
     if (mLoadTimer) {
@@ -687,3 +687,14 @@ nsUserFontSet::ReplaceFontEntry(gfxProxyFontEntry *aProxy,
   static_cast<gfxMixedFontFamily*>(aProxy->Family())->
     ReplaceFontEntry(aProxy, aFontEntry);
 }
+
+nsCSSFontFaceRule*
+nsUserFontSet::FindRuleForEntry(gfxFontEntry *aFontEntry)
+{
+  for (PRUint32 i = 0; i < mRules.Length(); ++i) {
+    if (mRules[i].mFontEntry == aFontEntry) {
+      return mRules[i].mContainer.mRule;
+    }
+  }
+  return nsnull;
+}
