@@ -43,7 +43,6 @@
 #include "nsPresContext.h"
 #include "nsCOMPtr.h"
 #include "nsIContent.h"
-#include "nsIPresShell.h"
 #include "nsHTMLContainerFrame.h"
 #include "nsINameSpaceManager.h"
 #include "nsGkAtoms.h"
@@ -51,13 +50,13 @@
 #include "nsIDOMNode.h"
 #include "nsIDOMNamedNodeMap.h"
 #include "nsIDOMAttr.h"
-#include "nsIWidget.h"
-#include "nsIRenderingContext.h"
 #include "nsIDocument.h"
-#include "nsIDeviceContext.h"
 #include "nsITheme.h"
 #include "nsIServiceManager.h"
 #include "nsIBoxLayout.h"
+#include "FrameLayerBuilder.h"
+
+using namespace mozilla;
 
 #ifdef DEBUG_LAYOUT
 PRInt32 gIndent = 0;
@@ -179,6 +178,12 @@ nsBox::BeginLayout(nsBoxLayoutState& aState)
       box->AddStateBits(NS_FRAME_IS_DIRTY);
   }
 
+  // Another copy-over from nsHTMLReflowState.
+  // Since we are in reflow, we don't need to store these properties anymore.
+  FrameProperties props = Properties();
+  props.Delete(UsedBorderProperty());
+  props.Delete(UsedPaddingProperty());
+  props.Delete(UsedMarginProperty());
 
 #ifdef DEBUG_LAYOUT
   PropagateDebug(aState);
@@ -261,7 +266,7 @@ nsIFrame::GetClientRect(nsRect& aClientRect)
 }
 
 void
-nsBox::SetBounds(nsBoxLayoutState& aState, const nsRect& aRect, PRBool aRemoveOverflowArea)
+nsBox::SetBounds(nsBoxLayoutState& aState, const nsRect& aRect, PRBool aRemoveOverflowAreas)
 {
     NS_BOX_ASSERTION(this, aRect.width >=0 && aRect.height >= 0, "SetBounds Size < 0");
 
@@ -281,11 +286,9 @@ nsBox::SetBounds(nsBoxLayoutState& aState, const nsRect& aRect, PRBool aRemoveOv
 
     // Nuke the overflow area. The caller is responsible for restoring
     // it if necessary.
-    if (aRemoveOverflowArea && (GetStateBits() & NS_FRAME_OUTSIDE_CHILDREN)) {
+    if (aRemoveOverflowAreas) {
       // remove the previously stored overflow area
-      PresContext()->PropertyTable()->
-        DeleteProperty(this, nsGkAtoms::overflowAreaProperty);
-      RemoveStateBits(NS_FRAME_OUTSIDE_CHILDREN);
+      ClearOverflowRects();
     }
 
     if (!(flags & NS_FRAME_NO_MOVE_VIEW))
@@ -344,7 +347,7 @@ nsBox::GetBorder(nsMargin& aMargin)
     // Go to the theme for the border.
     nsPresContext *context = PresContext();
     if (gTheme->ThemeSupportsWidget(context, this, disp->mAppearance)) {
-      nsMargin margin(0, 0, 0, 0);
+      nsIntMargin margin(0, 0, 0, 0);
       gTheme->GetWidgetBorder(context->DeviceContext(), this,
                               disp->mAppearance, &margin);
       aMargin.top = context->DevPixelsToAppUnits(margin.top);
@@ -368,7 +371,7 @@ nsBox::GetPadding(nsMargin& aMargin)
     // Go to the theme for the padding.
     nsPresContext *context = PresContext();
     if (gTheme->ThemeSupportsWidget(context, this, disp->mAppearance)) {
-      nsMargin margin(0, 0, 0, 0);
+      nsIntMargin margin(0, 0, 0, 0);
       PRBool useThemePadding;
 
       useThemePadding = gTheme->GetWidgetPadding(context->DeviceContext(),
@@ -407,7 +410,7 @@ nsBox::SizeNeedsRecalc(nsSize& aSize)
 }
 
 void
-nsBox::CoordNeedsRecalc(PRInt32& aFlex)
+nsBox::CoordNeedsRecalc(nscoord& aFlex)
 {
   aFlex = -1;
 }
@@ -424,35 +427,6 @@ nsBox::DoesNeedRecalc(nscoord aCoord)
   return (aCoord == -1);
 }
 
-PRBool
-nsBox::GetWasCollapsed(nsBoxLayoutState& aState)
-{
-  return (GetStateBits() & NS_STATE_IS_COLLAPSED) != 0;
-}
-
-void
-nsBox::SetWasCollapsed(nsBoxLayoutState& aState, PRBool aCollapsed)
-{
-  if (aCollapsed)
-     AddStateBits(NS_STATE_IS_COLLAPSED);
-  else
-     RemoveStateBits(NS_STATE_IS_COLLAPSED);
-}
-
-NS_IMETHODIMP
-nsBox::SetLayoutManager(nsIBoxLayout* aLayout)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsBox::GetLayoutManager(nsIBoxLayout** aLayout)
-{
-  *aLayout = nsnull;
-  return NS_OK;
-}
-
-
 nsSize
 nsBox::GetPrefSize(nsBoxLayoutState& aState)
 {
@@ -465,7 +439,8 @@ nsBox::GetPrefSize(nsBoxLayoutState& aState)
     return pref;
 
   AddBorderAndPadding(pref);
-  nsIBox::AddCSSPrefSize(aState, this, pref);
+  PRBool widthSet, heightSet;
+  nsIBox::AddCSSPrefSize(this, pref, widthSet, heightSet);
 
   nsSize minSize = GetMinSize(aState);
   nsSize maxSize = GetMaxSize(aState);
@@ -484,7 +459,8 @@ nsBox::GetMinSize(nsBoxLayoutState& aState)
     return min;
 
   AddBorderAndPadding(min);
-  nsIBox::AddCSSMinSize(aState, this, min);
+  PRBool widthSet, heightSet;
+  nsIBox::AddCSSMinSize(aState, this, min, widthSet, heightSet);
   return min;
 }
 
@@ -506,7 +482,8 @@ nsBox::GetMaxSize(nsBoxLayoutState& aState)
     return maxSize;
 
   AddBorderAndPadding(maxSize);
-  nsIBox::AddCSSMaxSize(aState, this, maxSize);
+  PRBool widthSet, heightSet;
+  nsIBox::AddCSSMaxSize(this, maxSize, widthSet, heightSet);
   return maxSize;
 }
 
@@ -515,7 +492,6 @@ nsBox::GetFlex(nsBoxLayoutState& aState)
 {
   nscoord flex = 0;
 
-  GetDefaultFlex(flex);
   nsIBox::AddCSSFlex(aState, this, flex);
 
   return flex;
@@ -524,8 +500,19 @@ nsBox::GetFlex(nsBoxLayoutState& aState)
 PRUint32
 nsIFrame::GetOrdinal(nsBoxLayoutState& aState)
 {
-  PRUint32 ordinal = DEFAULT_ORDINAL_GROUP;
-  nsIBox::AddCSSOrdinal(aState, this, ordinal);
+  PRUint32 ordinal = GetStyleXUL()->mBoxOrdinal;
+
+  // When present, attribute value overrides CSS.
+  nsIContent* content = GetContent();
+  if (content && content->IsXUL()) {
+    PRInt32 error;
+    nsAutoString value;
+
+    content->GetAttr(kNameSpaceID_None, nsGkAtoms::ordinal, value);
+    if (!value.IsEmpty()) {
+      ordinal = value.ToInteger(&error);
+    }
+  }
 
   return ordinal;
 }
@@ -542,10 +529,7 @@ nsBox::GetBoxAscent(nsBoxLayoutState& aState)
 PRBool
 nsBox::IsCollapsed(nsBoxLayoutState& aState)
 {
-  PRBool collapsed = PR_FALSE;
-  nsIBox::AddCSSCollapsed(aState, this, collapsed);
-
-  return collapsed;
+  return GetStyleVisibility()->mVisible == NS_STYLE_VISIBILITY_COLLAPSE;
 }
 
 nsresult
@@ -603,28 +587,29 @@ nsBox::SyncLayout(nsBoxLayoutState& aState)
 
   flags |= stateFlags;
 
-  nsRect rect(nsPoint(0, 0), GetSize());
+  nsRect visualOverflow;
 
   if (ComputesOwnOverflowArea()) {
-    rect = GetOverflowRect();
+    visualOverflow = GetVisualOverflowRect();
   }
   else {
+    nsRect rect(nsPoint(0, 0), GetSize());
+    nsOverflowAreas overflowAreas(rect, rect);
     if (!DoesClipChildren() && !IsCollapsed(aState)) {
       // See if our child frames caused us to overflow after being laid
       // out. If so, store the overflow area.  This normally can't happen
       // in XUL, but it can happen with the CSS 'outline' property and
       // possibly with other exotic stuff (e.g. relatively positioned
       // frames in HTML inside XUL).
-      nsIFrame* box = GetChildBox();
-      while (box) {
-        nsRect bounds = box->GetOverflowRect() + box->GetPosition();
-        rect.UnionRect(rect, bounds);
-
-        box = box->GetNextBox();
+      for (nsIFrame* kid = GetChildBox(); kid; kid = kid->GetNextBox()) {
+        nsOverflowAreas kidOverflow =
+          kid->GetOverflowAreas() + kid->GetPosition();
+        overflowAreas.UnionWith(kidOverflow);
       }
     }
 
-    FinishAndStoreOverflow(&rect, GetSize());
+    FinishAndStoreOverflow(overflowAreas, GetSize());
+    visualOverflow = overflowAreas.VisualOverflow();
   }
 
   nsIView* view = GetView();
@@ -635,7 +620,7 @@ nsBox::SyncLayout(nsBoxLayoutState& aState)
                              presContext, 
                              this,
                              view,
-                             &rect,
+                             visualOverflow,
                              flags);
   } 
 
@@ -644,8 +629,7 @@ nsBox::SyncLayout(nsBoxLayoutState& aState)
 
 nsresult
 nsIFrame::Redraw(nsBoxLayoutState& aState,
-                 const nsRect*   aDamageRect,
-                 PRBool          aImmediate)
+                 const nsRect*   aDamageRect)
 {
   if (aState.PaintingDisabled())
     return NS_OK;
@@ -654,17 +638,21 @@ nsIFrame::Redraw(nsBoxLayoutState& aState,
   if (aDamageRect)
     damageRect = *aDamageRect;
   else
-    damageRect = GetOverflowRect();
+    damageRect = GetVisualOverflowRect();
 
-  InvalidateWithFlags(damageRect, aImmediate ? INVALIDATE_IMMEDIATE : 0);
+  Invalidate(damageRect);
+  // nsStackLayout, at least, expects us to repaint descendants even
+  // if a damage rect is provided
+  FrameLayerBuilder::InvalidateThebesLayersInSubtree(this);
 
   return NS_OK;
 }
 
-PRBool 
-nsIBox::AddCSSPrefSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
+PRBool
+nsIBox::AddCSSPrefSize(nsIBox* aBox, nsSize& aSize, PRBool &aWidthSet, PRBool &aHeightSet)
 {
-    PRBool widthSet = PR_FALSE, heightSet = PR_FALSE;
+    aWidthSet = PR_FALSE;
+    aHeightSet = PR_FALSE;
 
     // add in the css min, max, pref
     const nsStylePosition* position = aBox->GetStylePosition();
@@ -674,21 +662,39 @@ nsIBox::AddCSSPrefSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     // (Handling the eStyleUnit_Enumerated types requires
     // GetPrefSize/GetMinSize methods that don't consider
     // (min-/max-/)(width/height) properties.)
-    if (position->mWidth.GetUnit() == eStyleUnit_Coord) {
-        aSize.width = position->mWidth.GetCoordValue();
-        widthSet = PR_TRUE;
+    const nsStyleCoord &width = position->mWidth;
+    if (width.GetUnit() == eStyleUnit_Coord) {
+        aSize.width = width.GetCoordValue();
+        aWidthSet = PR_TRUE;
+    } else if (width.IsCalcUnit()) {
+        if (!width.CalcHasPercent()) {
+            // pass 0 for percentage basis since we know there are no %s
+            aSize.width = nsRuleNode::ComputeComputedCalc(width, 0);
+            if (aSize.width < 0)
+                aSize.width = 0;
+            aWidthSet = PR_TRUE;
+        }
     }
 
-    if (position->mHeight.GetUnit() == eStyleUnit_Coord) {
-        aSize.height = position->mHeight.GetCoordValue();     
-        heightSet = PR_TRUE;
+    const nsStyleCoord &height = position->mHeight;
+    if (height.GetUnit() == eStyleUnit_Coord) {
+        aSize.height = height.GetCoordValue();
+        aHeightSet = PR_TRUE;
+    } else if (height.IsCalcUnit()) {
+        if (!height.CalcHasPercent()) {
+            // pass 0 for percentage basis since we know there are no %s
+            aSize.height = nsRuleNode::ComputeComputedCalc(height, 0);
+            if (aSize.height < 0)
+                aSize.height = 0;
+            aHeightSet = PR_TRUE;
+        }
     }
-    
+
     nsIContent* content = aBox->GetContent();
     // ignore 'height' and 'width' attributes if the actual element is not XUL
     // For example, we might be magic XUL frames whose primary content is an HTML
     // <select>
-    if (content && content->IsNodeOfType(nsINode::eXUL)) {
+    if (content && content->IsXUL()) {
         nsAutoString value;
         PRInt32 error;
 
@@ -698,7 +704,7 @@ nsIBox::AddCSSPrefSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
 
             aSize.width =
               nsPresContext::CSSPixelsToAppUnits(value.ToInteger(&error));
-            widthSet = PR_TRUE;
+            aWidthSet = PR_TRUE;
         }
 
         content->GetAttr(kNameSpaceID_None, nsGkAtoms::height, value);
@@ -707,20 +713,21 @@ nsIBox::AddCSSPrefSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
 
             aSize.height =
               nsPresContext::CSSPixelsToAppUnits(value.ToInteger(&error));
-            heightSet = PR_TRUE;
+            aHeightSet = PR_TRUE;
         }
     }
 
-    return (widthSet && heightSet);
+    return (aWidthSet && aHeightSet);
 }
 
 
-PRBool 
-nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
+PRBool
+nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize,
+                      PRBool &aWidthSet, PRBool &aHeightSet)
 {
+    aWidthSet = PR_FALSE;
+    aHeightSet = PR_FALSE;
 
-    PRBool widthSet = PR_FALSE;
-    PRBool heightSet = PR_FALSE;
     PRBool canOverride = PR_TRUE;
 
     // See if a native theme wants to supply a minimum size.
@@ -728,18 +735,18 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     if (display->mAppearance) {
       nsITheme *theme = aState.PresContext()->GetTheme();
       if (theme && theme->ThemeSupportsWidget(aState.PresContext(), aBox, display->mAppearance)) {
-        nsSize size;
-        nsIRenderingContext* rendContext = aState.GetRenderingContext();
+        nsIntSize size;
+        nsRenderingContext* rendContext = aState.GetRenderingContext();
         if (rendContext) {
           theme->GetMinimumWidgetSize(rendContext, aBox,
                                       display->mAppearance, &size, &canOverride);
           if (size.width) {
             aSize.width = aState.PresContext()->DevPixelsToAppUnits(size.width);
-            widthSet = PR_TRUE;
+            aWidthSet = PR_TRUE;
           }
           if (size.height) {
             aSize.height = aState.PresContext()->DevPixelsToAppUnits(size.height);
-            heightSet = PR_TRUE;
+            aHeightSet = PR_TRUE;
           }
         }
       }
@@ -749,39 +756,49 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     const nsStylePosition* position = aBox->GetStylePosition();
 
     // same for min size. Unfortunately min size is always set to 0. So for now
-    // we will assume 0 means not set.
-    if (position->mMinWidth.GetUnit() == eStyleUnit_Coord) {
-        nscoord min = position->mMinWidth.GetCoordValue();
-        if (min && (!widthSet || (min > aSize.width && canOverride))) {
+    // we will assume 0 (as a coord) means not set.
+    const nsStyleCoord &minWidth = position->mMinWidth;
+    if ((minWidth.GetUnit() == eStyleUnit_Coord &&
+         minWidth.GetCoordValue() != 0) ||
+        (minWidth.IsCalcUnit() && !minWidth.CalcHasPercent())) {
+        nscoord min = nsRuleNode::ComputeCoordPercentCalc(minWidth, 0);
+        if (!aWidthSet || (min > aSize.width && canOverride)) {
            aSize.width = min;
-           widthSet = PR_TRUE;
+           aWidthSet = PR_TRUE;
         }
-    } else if (position->mMinWidth.GetUnit() == eStyleUnit_Percent) {
-        NS_ASSERTION(position->mMinWidth.GetPercentValue() == 0.0f,
+    } else if (minWidth.GetUnit() == eStyleUnit_Percent) {
+        NS_ASSERTION(minWidth.GetPercentValue() == 0.0f,
           "Non-zero percentage values not currently supported");
         aSize.width = 0;
-        widthSet = PR_TRUE;
+        aWidthSet = PR_TRUE; // FIXME: should we really do this for
+                             // nonzero values?
     }
     // XXX Handle eStyleUnit_Enumerated?
     // (Handling the eStyleUnit_Enumerated types requires
     // GetPrefSize/GetMinSize methods that don't consider
     // (min-/max-/)(width/height) properties.
+    // calc() with percentage is treated like '0' (unset)
 
-    if (position->mMinHeight.GetUnit() == eStyleUnit_Coord) {
-        nscoord min = position->mMinHeight.GetCoordValue();
-        if (min && (!heightSet || (min > aSize.height && canOverride))) {
+    const nsStyleCoord &minHeight = position->mMinHeight;
+    if ((minHeight.GetUnit() == eStyleUnit_Coord &&
+         minHeight.GetCoordValue() != 0) ||
+        (minHeight.IsCalcUnit() && !minHeight.CalcHasPercent())) {
+        nscoord min = nsRuleNode::ComputeCoordPercentCalc(minHeight, 0);
+        if (!aHeightSet || (min > aSize.height && canOverride)) {
            aSize.height = min;
-           heightSet = PR_TRUE;
+           aHeightSet = PR_TRUE;
         }
-    } else if (position->mMinHeight.GetUnit() == eStyleUnit_Percent) {
+    } else if (minHeight.GetUnit() == eStyleUnit_Percent) {
         NS_ASSERTION(position->mMinHeight.GetPercentValue() == 0.0f,
           "Non-zero percentage values not currently supported");
         aSize.height = 0;
-        heightSet = PR_TRUE;
+        aHeightSet = PR_TRUE; // FIXME: should we really do this for
+                              // nonzero values?
     }
+    // calc() with percentage is treated like '0' (unset)
 
     nsIContent* content = aBox->GetContent();
-    if (content) {
+    if (content && content->IsXUL()) {
         nsAutoString value;
         PRInt32 error;
 
@@ -794,7 +811,7 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
               nsPresContext::CSSPixelsToAppUnits(value.ToInteger(&error));
             if (val > aSize.width)
               aSize.width = val;
-            widthSet = PR_TRUE;
+            aWidthSet = PR_TRUE;
         }
 
         content->GetAttr(kNameSpaceID_None, nsGkAtoms::minheight, value);
@@ -807,17 +824,18 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
             if (val > aSize.height)
               aSize.height = val;
 
-            heightSet = PR_TRUE;
+            aHeightSet = PR_TRUE;
         }
     }
 
-    return (widthSet && heightSet);
+    return (aWidthSet && aHeightSet);
 }
 
-PRBool 
-nsIBox::AddCSSMaxSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
-{  
-    PRBool widthSet = PR_FALSE, heightSet = PR_FALSE;
+PRBool
+nsIBox::AddCSSMaxSize(nsIBox* aBox, nsSize& aSize, PRBool &aWidthSet, PRBool &aHeightSet)
+{
+    aWidthSet = PR_FALSE;
+    aHeightSet = PR_FALSE;
 
     // add in the css min, max, pref
     const nsStylePosition* position = aBox->GetStylePosition();
@@ -828,18 +846,22 @@ nsIBox::AddCSSMaxSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     // (Handling the eStyleUnit_Enumerated types requires
     // GetPrefSize/GetMinSize methods that don't consider
     // (min-/max-/)(width/height) properties.)
-    if (position->mMaxWidth.GetUnit() == eStyleUnit_Coord) {
-        aSize.width = position->mMaxWidth.GetCoordValue();
-        widthSet = PR_TRUE;
+    const nsStyleCoord maxWidth = position->mMaxWidth;
+    if (maxWidth.ConvertsToLength()) {
+        aSize.width = nsRuleNode::ComputeCoordPercentCalc(maxWidth, 0);
+        aWidthSet = PR_TRUE;
     }
+    // percentages and calc() with percentages are treated like 'none'
 
-    if (position->mMaxHeight.GetUnit() == eStyleUnit_Coord) {
-        aSize.height = position->mMaxHeight.GetCoordValue();
-        heightSet = PR_TRUE;
+    const nsStyleCoord &maxHeight = position->mMaxHeight;
+    if (maxHeight.ConvertsToLength()) {
+        aSize.height = nsRuleNode::ComputeCoordPercentCalc(maxHeight, 0);
+        aHeightSet = PR_TRUE;
     }
+    // percentages and calc() with percentages are treated like 'none'
 
     nsIContent* content = aBox->GetContent();
-    if (content) {
+    if (content && content->IsXUL()) {
         nsAutoString value;
         PRInt32 error;
 
@@ -850,7 +872,7 @@ nsIBox::AddCSSMaxSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
             nscoord val =
               nsPresContext::CSSPixelsToAppUnits(value.ToInteger(&error));
             aSize.width = val;
-            widthSet = PR_TRUE;
+            aWidthSet = PR_TRUE;
         }
 
         content->GetAttr(kNameSpaceID_None, nsGkAtoms::maxheight, value);
@@ -861,21 +883,24 @@ nsIBox::AddCSSMaxSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
               nsPresContext::CSSPixelsToAppUnits(value.ToInteger(&error));
             aSize.height = val;
 
-            heightSet = PR_TRUE;
+            aHeightSet = PR_TRUE;
         }
     }
 
-    return (widthSet || heightSet);
+    return (aWidthSet || aHeightSet);
 }
 
-PRBool 
+PRBool
 nsIBox::AddCSSFlex(nsBoxLayoutState& aState, nsIBox* aBox, nscoord& aFlex)
 {
     PRBool flexSet = PR_FALSE;
 
     // get the flexibility
+    aFlex = aBox->GetStyleXUL()->mBoxFlex;
+
+    // attribute value overrides CSS
     nsIContent* content = aBox->GetContent();
-    if (content) {
+    if (content && content->IsXUL()) {
         PRInt32 error;
         nsAutoString value;
 
@@ -885,15 +910,6 @@ nsIBox::AddCSSFlex(nsBoxLayoutState& aState, nsIBox* aBox, nscoord& aFlex)
             aFlex = value.ToInteger(&error);
             flexSet = PR_TRUE;
         }
-        else {
-          // No attribute value.  Check CSS.
-          const nsStyleXUL* boxInfo = aBox->GetStyleXUL();
-          if (boxInfo->mBoxFlex > 0.0f) {
-            // The flex was defined in CSS.
-            aFlex = (nscoord)boxInfo->mBoxFlex;
-            flexSet = PR_TRUE;
-          }
-        }
     }
 
     if (aFlex < 0)
@@ -901,45 +917,7 @@ nsIBox::AddCSSFlex(nsBoxLayoutState& aState, nsIBox* aBox, nscoord& aFlex)
     if (aFlex >= nscoord_MAX)
       aFlex = nscoord_MAX - 1;
 
-    return flexSet;
-}
-
-PRBool 
-nsIBox::AddCSSCollapsed(nsBoxLayoutState& aState, nsIBox* aBox, PRBool& aCollapsed)
-{
-  aCollapsed = aBox->GetStyleVisibility()->mVisible ==
-               NS_STYLE_VISIBILITY_COLLAPSE;
-  return PR_TRUE;
-}
-
-PRBool 
-nsIBox::AddCSSOrdinal(nsBoxLayoutState& aState, nsIBox* aBox, PRUint32& aOrdinal)
-{
-  PRBool ordinalSet = PR_FALSE;
-  
-  // get the flexibility
-  nsIContent* content = aBox->GetContent();
-  if (content) {
-    PRInt32 error;
-    nsAutoString value;
-
-    content->GetAttr(kNameSpaceID_None, nsGkAtoms::ordinal, value);
-    if (!value.IsEmpty()) {
-      aOrdinal = value.ToInteger(&error);
-      ordinalSet = PR_TRUE;
-    }
-    else {
-      // No attribute value.  Check CSS.
-      const nsStyleXUL* boxInfo = aBox->GetStyleXUL();
-      if (boxInfo->mBoxOrdinal > 1) {
-        // The ordinal group was defined in CSS.
-        aOrdinal = (nscoord)boxInfo->mBoxOrdinal;
-        ordinalSet = PR_TRUE;
-      }
-    }
-  }
-
-  return ordinalSet;
+    return flexSet || aFlex > 0;
 }
 
 void
@@ -948,12 +926,6 @@ nsBox::AddBorderAndPadding(nsSize& aSize)
   AddBorderAndPadding(this, aSize);
 }
 
-void
-nsBox::AddMargin(nsSize& aSize)
-{
-  AddMargin(this, aSize);
-}
-
 void
 nsBox::AddBorderAndPadding(nsIBox* aBox, nsSize& aSize)
 {
@@ -995,8 +967,8 @@ nsBox::BoundsCheck(nscoord aMin, nscoord aPref, nscoord aMax)
 nsSize
 nsBox::BoundsCheckMinMax(const nsSize& aMinSize, const nsSize& aMaxSize)
 {
-  return nsSize(PR_MAX(aMaxSize.width, aMinSize.width),
-                PR_MAX(aMaxSize.height, aMinSize.height));
+  return nsSize(NS_MAX(aMaxSize.width, aMinSize.width),
+                NS_MAX(aMaxSize.height, aMinSize.height));
 }
 
 nsSize
@@ -1051,19 +1023,3 @@ nsBox::GetDebug(PRBool& aDebug)
 }
 
 #endif
-
-PRBool
-nsBox::GetMouseThrough() const
-{
-  if (mParent && mParent->IsBoxFrame())
-    return mParent->GetMouseThrough();
-
-  return PR_FALSE;
-}
-
-PRBool
-nsBox::GetDefaultFlex(PRInt32& aFlex) 
-{ 
-  aFlex = 0; 
-  return PR_TRUE; 
-}
