@@ -696,6 +696,10 @@ cryptojs_generateOneKeyPair(JSContext *cx, nsKeyPairInfo *keyPairInfo,
                                                      (params) ? strlen(params):0, 
                                                      keySize, keyPairInfo);
 
+  if (!keyGenParams) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
   // Make sure the token has password already set on it before trying
   // to generate the key.
 
@@ -870,8 +874,7 @@ cryptojs_ReadArgsAndGenerateKey(JSContext *cx,
                                 PK11SlotInfo **slot, PRBool willEscrow)
 {
   JSString  *jsString;
-  char    *params, *keyGenAlg; //Never free these strings cause
-                               //they are owned by the JS layer.
+  JSAutoByteString params, keyGenAlg;
   int    keySize;
   nsresult  rv;
 
@@ -881,13 +884,12 @@ cryptojs_ReadArgsAndGenerateKey(JSContext *cx,
     return NS_ERROR_FAILURE;
   }
   keySize = JSVAL_TO_INT(argv[0]);
-  if (JSVAL_IS_NULL(argv[1])) {
-    params = nsnull;
-  } else {
+  if (!JSVAL_IS_NULL(argv[1])) {
     jsString = JS_ValueToString(cx,argv[1]);
     NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
     argv[1] = STRING_TO_JSVAL(jsString);
-    params   = JS_GetStringBytes(jsString);
+    params.encode(cx, jsString);
+    NS_ENSURE_TRUE(!!params, NS_ERROR_OUT_OF_MEMORY);
   }
 
   if (JSVAL_IS_NULL(argv[2])) {
@@ -898,12 +900,13 @@ cryptojs_ReadArgsAndGenerateKey(JSContext *cx,
   jsString = JS_ValueToString(cx, argv[2]);
   NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
   argv[2] = STRING_TO_JSVAL(jsString);
-  keyGenAlg = JS_GetStringBytes(jsString);
-  keyGenType->keyGenType = cryptojs_interpret_key_gen_type(keyGenAlg);
+  keyGenAlg.encode(cx, jsString);
+  NS_ENSURE_TRUE(!!keyGenAlg, NS_ERROR_OUT_OF_MEMORY);
+  keyGenType->keyGenType = cryptojs_interpret_key_gen_type(keyGenAlg.ptr());
   if (keyGenType->keyGenType == invalidKeyGen) {
     JS_ReportError(cx, "%s%s%s", JS_ERROR,
                    "invalid key generation argument:",
-                   keyGenAlg);
+                   keyGenAlg.ptr());
     goto loser;
   }
   if (*slot == nsnull) {
@@ -912,13 +915,13 @@ cryptojs_ReadArgsAndGenerateKey(JSContext *cx,
       goto loser;
   }
 
-  rv = cryptojs_generateOneKeyPair(cx,keyGenType,keySize,params,uiCxt,*slot,
-                                   willEscrow);
+  rv = cryptojs_generateOneKeyPair(cx,keyGenType,keySize,params.ptr(),uiCxt,
+                                   *slot,willEscrow);
 
   if (rv != NS_OK) {
     JS_ReportError(cx,"%s%s%s", JS_ERROR,
                    "could not generate the key for algorithm ",
-                   keyGenAlg);
+                   keyGenAlg.ptr());
     goto loser;
   }
   return NS_OK;
@@ -1849,37 +1852,32 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
   JSString *jsString = JS_ValueToString(cx,argv[0]);
   NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
   argv[0] = STRING_TO_JSVAL(jsString);
-  
-  char * reqDN = JS_GetStringBytes(jsString);
-  char *regToken;
-  if (JSVAL_IS_NULL(argv[1])) {
-    regToken           = nsnull;
-  } else {
+  JSAutoByteString reqDN(cx,jsString);
+  NS_ENSURE_TRUE(!!reqDN, NS_ERROR_OUT_OF_MEMORY);
+
+  JSAutoByteString regToken;
+  if (!JSVAL_IS_NULL(argv[1])) {
     jsString = JS_ValueToString(cx, argv[1]);
     NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
     argv[1] = STRING_TO_JSVAL(jsString);
-
-    regToken = JS_GetStringBytes(jsString);
+    regToken.encode(cx, jsString);
+    NS_ENSURE_TRUE(!!regToken, NS_ERROR_OUT_OF_MEMORY);
   }
-  char *authenticator;
-  if (JSVAL_IS_NULL(argv[2])) {
-    authenticator           = nsnull;
-  } else {
+  JSAutoByteString authenticator;
+  if (!JSVAL_IS_NULL(argv[2])) {
     jsString      = JS_ValueToString(cx, argv[2]);
     NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
     argv[2] = STRING_TO_JSVAL(jsString);
-
-    authenticator = JS_GetStringBytes(jsString);
+    authenticator.encode(cx, jsString);
+    NS_ENSURE_TRUE(!!authenticator, NS_ERROR_OUT_OF_MEMORY);
   }
-  char *eaCert;
-  if (JSVAL_IS_NULL(argv[3])) {
-    eaCert           = nsnull;
-  } else {
+  JSAutoByteString eaCert;
+  if (!JSVAL_IS_NULL(argv[3])) {
     jsString     = JS_ValueToString(cx, argv[3]);
     NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
     argv[3] = STRING_TO_JSVAL(jsString);
-
-    eaCert       = JS_GetStringBytes(jsString);
+    eaCert.encode(cx, jsString);
+    NS_ENSURE_TRUE(!!eaCert, NS_ERROR_OUT_OF_MEMORY);
   }
   if (JSVAL_IS_NULL(argv[4])) {
     JS_ReportError(cx, "%s%s\n", JS_ERROR, "no completion "
@@ -1889,9 +1887,8 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
   jsString = JS_ValueToString(cx, argv[4]);
   NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
   argv[4] = STRING_TO_JSVAL(jsString);
-
-  char *jsCallback = JS_GetStringBytes(jsString);
-
+  JSAutoByteString jsCallback(cx, jsString);
+  NS_ENSURE_TRUE(!!jsCallback, NS_ERROR_OUT_OF_MEMORY);
 
   nrv = xpc->WrapNative(cx, ::JS_GetGlobalObject(cx),
                         static_cast<nsIDOMCrypto *>(this),
@@ -1908,9 +1905,9 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
   nsNSSCertificate *escrowCert = nsnull;
   nsCOMPtr<nsIX509Cert> nssCert;
   PRBool willEscrow = PR_FALSE;
-  if (eaCert) {
+  if (!!eaCert) {
     SECItem certDer = {siBuffer, nsnull, 0};
-    SECStatus srv = ATOB_ConvertAsciiToItem(&certDer, eaCert);
+    SECStatus srv = ATOB_ConvertAsciiToItem(&certDer, eaCert.ptr());
     if (srv != SECSuccess) {
       return NS_ERROR_FAILURE;
     }
@@ -1920,7 +1917,7 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
     if (!cert)
       return NS_ERROR_FAILURE;
 
-    escrowCert = new nsNSSCertificate(cert);
+    escrowCert = nsNSSCertificate::Create(cert);
     CERT_DestroyCertificate(cert);
     nssCert = escrowCert;
     if (!nssCert)
@@ -1975,9 +1972,10 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
   if (slot) 
     PK11_FreeSlot(slot);
 
-  char *encodedRequest = nsCreateReqFromKeyPairs(keyids, numRequests,
-                                                 reqDN, regToken, 
-                                                 authenticator,escrowCert);
+  char *encodedRequest = nsCreateReqFromKeyPairs(keyids,numRequests,
+                                                 reqDN.ptr(),regToken.ptr(),
+                                                 authenticator.ptr(),
+                                                 escrowCert);
 #ifdef DEBUG_javi
   printf ("Created the folloing CRMF request:\n%s\n", encodedRequest);
 #endif
@@ -2015,7 +2013,8 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
   NS_ENSURE_TRUE(secMan, NS_ERROR_UNEXPECTED);
   
   nsCOMPtr<nsIPrincipal> principals;
-  secMan->GetSubjectPrincipal(getter_AddRefs(principals));
+  nsresult rv = secMan->GetSubjectPrincipal(getter_AddRefs(principals));
+  NS_ENSURE_SUCCESS(rv, rv);
   NS_ENSURE_TRUE(principals, NS_ERROR_UNEXPECTED);
   
   nsCryptoRunArgs *args = new nsCryptoRunArgs();
@@ -2025,14 +2024,15 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
   args->m_cx         = cx;
   args->m_kungFuDeathGrip = GetISupportsFromContext(cx);
   args->m_scope      = JS_GetParent(cx, script_obj);
-  args->m_jsCallback.Adopt(jsCallback ? nsCRT::strdup(jsCallback) : 0);
+
+  args->m_jsCallback.Adopt(!!jsCallback ? nsCRT::strdup(jsCallback.ptr()) : 0);
   args->m_principals = principals;
   
   nsCryptoRunnable *cryptoRunnable = new nsCryptoRunnable(args);
   if (!cryptoRunnable)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  nsresult rv = NS_DispatchToMainThread(cryptoRunnable);
+  rv = NS_DispatchToMainThread(cryptoRunnable);
   if (NS_FAILED(rv))
     delete cryptoRunnable;
 
@@ -2108,7 +2108,7 @@ nsP12Runnable::Run()
   nsCOMPtr<nsIFilePicker> filePicker = 
                         do_CreateInstance("@mozilla.org/filepicker;1", &rv);
   if (!filePicker) {
-    NS_ASSERTION(0, "Could not create a file picker when backing up certs.");
+    NS_ERROR("Could not create a file picker when backing up certs.");
     return rv;
   }
 
@@ -2159,7 +2159,7 @@ nsCryptoRunnable::nsCryptoRunnable(nsCryptoRunArgs *args)
   NS_ASSERTION(args,"Passed nsnull to nsCryptoRunnable constructor.");
   m_args = args;
   NS_IF_ADDREF(m_args);
-  JS_AddNamedRoot(args->m_cx, &args->m_scope,"nsCryptoRunnable::mScope");
+  JS_AddNamedObjectRoot(args->m_cx, &args->m_scope,"nsCryptoRunnable::mScope");
 }
 
 nsCryptoRunnable::~nsCryptoRunnable()
@@ -2168,7 +2168,7 @@ nsCryptoRunnable::~nsCryptoRunnable()
 
   {
     JSAutoRequest ar(m_args->m_cx);
-    JS_RemoveRoot(m_args->m_cx, &m_args->m_scope);
+    JS_RemoveObjectRoot(m_args->m_cx, &m_args->m_scope);
   }
 
   NS_IF_RELEASE(m_args);
@@ -2183,6 +2183,13 @@ nsCryptoRunnable::Run()
   JSPrincipals *principals;
   JSContext *cx = m_args->m_cx;
 
+  JSAutoRequest ar(cx);
+  JSAutoEnterCompartment ac;
+
+  if (!ac.enter(cx, m_args->m_scope)) {
+    return NS_ERROR_FAILURE;
+  }
+
   nsresult rv = m_args->m_principals->GetJSPrincipals(cx, &principals);
   if (NS_FAILED(rv))
     return NS_ERROR_FAILURE;
@@ -2190,11 +2197,10 @@ nsCryptoRunnable::Run()
   // make sure the right context is on the stack. must not return w/out popping
   nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1"));
   if (!stack || NS_FAILED(stack->Push(cx))) {
+    JSPRINCIPALS_DROP(cx, principals);
     return NS_ERROR_FAILURE;
   }
 
-  JSAutoRequest ar(cx);
-
   jsval retval;
   if (JS_EvaluateScriptForPrincipals(cx, m_args->m_scope, principals,
                                      m_args->m_jsCallback, 
@@ -2205,6 +2211,7 @@ nsCryptoRunnable::Run()
   }
 
   stack->Pop(nsnull);
+  JSPRINCIPALS_DROP(cx, principals);
   return rv;
 }
 
@@ -2214,15 +2221,9 @@ static PRBool
 nsCertAlreadyExists(SECItem *derCert)
 {
   CERTCertDBHandle *handle = CERT_GetDefaultCertDB();
-  PRArenaPool *arena;
   CERTCertificate *cert;
   PRBool retVal = PR_FALSE;
 
-  arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
-  NS_ASSERTION(arena, "Couldn't allocate an arena!");
-  if (!arena)
-    return PR_FALSE; //What else could we return?
-
   cert = CERT_FindCertByDERCert(handle, derCert);
   if (cert) {
     if (cert->isperm && !cert->nickname && !cert->emailAddr) {
@@ -2334,7 +2335,9 @@ nsCrypto::ImportUserCertificates(const nsAString& aNickname,
 
     if (nsCertAlreadyExists(&currCert->derCert)) {
       if (aDoForcedBackup) {
-        certArr[i] = new nsNSSCertificate(currCert);
+        certArr[i] = nsNSSCertificate::Create(currCert);
+        if (!certArr[i])
+          goto loser;
         NS_ADDREF(certArr[i]);
       }
       CERT_DestroyCertificate(currCert);
@@ -2365,7 +2368,9 @@ nsCrypto::ImportUserCertificates(const nsAString& aNickname,
       goto loser;
     }
     if (aDoForcedBackup) {
-      certArr[i] = new nsNSSCertificate(currCert);
+      certArr[i] = nsNSSCertificate::Create(currCert);
+      if (!certArr[i])
+        goto loser;
       NS_ADDREF(certArr[i]);
     }
     CERT_DestroyCertificate(currCert);
@@ -2553,16 +2558,15 @@ nsCrypto::SignText(const nsAString& aStringToSign, const nsAString& aCaOption,
 
   PRUint32 numCAs = argc - 2;
   if (numCAs > 0) {
-    nsAutoArrayPtr<char*> caNames(new char*[numCAs]);
-    if (!caNames) {
-      aResult.Append(internalError);
-
-      return NS_OK;
-    }
-
     jsval *argv = nsnull;
     ncc->GetArgvPtr(&argv);
 
+    nsAutoArrayPtr<JSAutoByteString> caNameBytes(new JSAutoByteString[numCAs]);
+    if (!caNameBytes) {
+      aResult.Append(internalError);
+      return NS_OK;
+    }
+
     JSAutoRequest ar(cx);
 
     PRUint32 i;
@@ -2570,15 +2574,19 @@ nsCrypto::SignText(const nsAString& aStringToSign, const nsAString& aCaOption,
       JSString *caName = JS_ValueToString(cx, argv[i]);
       NS_ENSURE_TRUE(caName, NS_ERROR_OUT_OF_MEMORY);
       argv[i] = STRING_TO_JSVAL(caName);
-
-      if (!caName) {
-        aResult.Append(internalError);
-
-        return NS_OK;
-      }
-      caNames[i - 2] = JS_GetStringBytes(caName);
+      caNameBytes[i - 2].encode(cx, caName);
+      NS_ENSURE_TRUE(!!caNameBytes[i - 2], NS_ERROR_OUT_OF_MEMORY);
     }
 
+    nsAutoArrayPtr<char*> caNames(new char*[numCAs]);
+    if (!caNames) {
+      aResult.Append(internalError);
+      return NS_OK;
+    }
+
+    for (i = 0; i < numCAs; ++i)
+      caNames[i] = caNameBytes[i].ptr();
+
     if (certList &&
         CERT_FilterCertListByCANames(certList, numCAs, caNames,
                                      certUsageEmailSigner) != SECSuccess) {
@@ -2670,7 +2678,7 @@ nsCrypto::SignText(const nsAString& aStringToSign, const nsAString& aCaOption,
   for (node = CERT_LIST_HEAD(certList), certsToUse = 0;
        !CERT_LIST_END(node, certList) && certsToUse < nicknames->numnicknames;
        node = CERT_LIST_NEXT(node)) {
-    nsRefPtr<nsNSSCertificate> tempCert = new nsNSSCertificate(node->cert);
+    nsRefPtr<nsNSSCertificate> tempCert = nsNSSCertificate::Create(node->cert);
     if (tempCert) {
       nsAutoString nickWithSerial, details;
       rv = tempCert->FormatUIStrings(NS_ConvertUTF8toUTF16(nicknames->nicknames[certsToUse]),
@@ -2930,7 +2938,9 @@ confirm_user(const PRUnichar *message)
   if (prompter) {
     nsPSMUITracker tracker;
     if (!tracker.isUIForbidden()) {
-      PRBool checkState;
+      // The actual value is irrelevant but we shouldn't be handing out
+      // malformed JSBools to XPConnect.
+      PRBool checkState = PR_FALSE;
       prompter->ConfirmEx(0, message,
                           (nsIPrompt::BUTTON_DELAY_ENABLE) +
                           (nsIPrompt::BUTTON_POS_1_DEFAULT) +
@@ -3014,7 +3024,7 @@ nsPkcs11::AddModule(const nsAString& aModuleName,
   case -2:
     return NS_ERROR_ILLEGAL_VALUE;
   }
-  NS_ASSERTION(0,"Bogus return value, this should never happen");
+  NS_ERROR("Bogus return value, this should never happen");
   return NS_ERROR_FAILURE;
 }
 
