@@ -386,11 +386,16 @@ int difftool(Options* inOptions)
             char* theLine = &lineBuffer[2];
             int scanRes = 0;
             int size;
-            char segClass[0x10];
-            char scope[0x10];
-            char module[0x100];
-            char segment[0x40];
-            char object[0x100];
+            #define SEGCLASS_CHARS 15
+            char segClass[SEGCLASS_CHARS + 1];
+            #define SCOPE_CHARS 15
+            char scope[SCOPE_CHARS + 1];
+            #define MODULE_CHARS 255
+            char module[MODULE_CHARS + 1];
+            #define SEGMENT_CHARS 63
+            char segment[SEGMENT_CHARS + 1];
+            #define OBJECT_CHARS 255
+            char object[OBJECT_CHARS + 1];
             char* symbol = NULL;
 
             /*
@@ -405,8 +410,15 @@ int difftool(Options* inOptions)
             /*
             **  Scan the line for information.
             */
+
+#define STRINGIFY(s_) STRINGIFY2(s_)
+#define STRINGIFY2(s_) #s_
+
             scanRes = sscanf(theLine,
-                "%x\t%s\t%s\t%s\t%s\t%s\t",
+                "%x\t%" STRINGIFY(SEGCLASS_CHARS) "s\t%"
+                STRINGIFY(SCOPE_CHARS) "s\t%" STRINGIFY(MODULE_CHARS)
+                "s\t%" STRINGIFY(SEGMENT_CHARS) "s\t%"
+                STRINGIFY(OBJECT_CHARS) "s\t",
                 (unsigned*)&size,
                 segClass,
                 scope,
