@@ -49,6 +49,10 @@
 #include <gtk/gtk.h>
 #endif
 
+#ifdef ANDROID
+#include "AndroidBridge.h"
+#endif
+
 #include "mozilla/Services.h"
 
 #include "nsConsoleMessage.h"
@@ -485,6 +489,10 @@ ParseManifestCommon(NSLocationType aType, nsILocalFile* aFile,
   nsTextFormatter::ssprintf(osVersion, NS_LITERAL_STRING("%ld.%ld").get(),
                                        gtk_major_version,
                                        gtk_minor_version);
+#elif defined(ANDROID)
+  if (mozilla::AndroidBridge::Bridge()) {
+    mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build$VERSION", "RELEASE", osVersion);
+  }
 #endif
 
   // Because contracts must be registered after CIDs, we save and process them
