@@ -51,7 +51,7 @@ using namespace mozilla;
 static int DEBUG_TotalInfos = 0;
 static int DEBUG_CurrentInfos = 0;
 static int DEBUG_MaxInfos = 0;
-static int DEBUG_MonitorEntryCount = 0;
+static int DEBUG_ReentrantMonitorEntryCount = 0;
 
 #define LOG_INFO_CREATE(t)                                                  \
     DEBUG_TotalInfos++;                                                     \
@@ -63,7 +63,7 @@ static int DEBUG_MonitorEntryCount = 0;
     DEBUG_CurrentInfos-- /* no ';' */
 
 #define LOG_INFO_MONITOR_ENTRY                                              \
-    DEBUG_MonitorEntryCount++ /* no ';' */
+    DEBUG_ReentrantMonitorEntryCount++ /* no ';' */
 
 #else /* SHOW_INFO_COUNT_STATS */
 
@@ -619,7 +619,10 @@ xptiInterfaceEntry::HasAncestor(const nsIID * iid, PRBool *_retval)
 nsresult 
 xptiInterfaceEntry::GetInterfaceInfo(xptiInterfaceInfo** info)
 {
-    MonitorAutoEnter lock(xptiInterfaceInfoManager::GetInfoMonitor());
+#ifdef DEBUG
+    xptiInterfaceInfoManager::GetSingleton()->GetWorkingSet()->mTableReentrantMonitor.
+        AssertCurrentThreadIn();
+#endif
     LOG_INFO_MONITOR_ENTRY;
 
     if(!mInfo)
@@ -646,6 +649,19 @@ xptiInterfaceEntry::LockedInvalidateInterfaceInfo()
     }
 }
 
+PRBool
+xptiInterfaceInfo::BuildParent()
+{
+    mozilla::ReentrantMonitorAutoEnter monitor(xptiInterfaceInfoManager::GetSingleton()->
+                                    GetWorkingSet()->mTableReentrantMonitor);
+    NS_ASSERTION(mEntry && 
+                 mEntry->IsFullyResolved() && 
+                 !mParent &&
+                 mEntry->Parent(),
+                "bad BuildParent call");
+    return NS_SUCCEEDED(mEntry->Parent()->GetInterfaceInfo(&mParent));
+}
+
 /***************************************************************************/
 
 NS_IMPL_QUERY_INTERFACE1(xptiInterfaceInfo, nsIInterfaceInfo)
@@ -679,9 +695,11 @@ xptiInterfaceInfo::Release(void)
     NS_LOG_RELEASE(this, cnt, "xptiInterfaceInfo");
     if(!cnt)
     {
-        MonitorAutoEnter lock(xptiInterfaceInfoManager::GetInfoMonitor());
+        mozilla::ReentrantMonitorAutoEnter monitor(xptiInterfaceInfoManager::
+                                          GetSingleton()->GetWorkingSet()->
+                                          mTableReentrantMonitor);
         LOG_INFO_MONITOR_ENTRY;
-        
+
         // If GetInterfaceInfo added and *released* a reference before we 
         // acquired the monitor then 'this' might already be dead. In that
         // case we would not want to try to access any instance data. We
@@ -695,7 +713,7 @@ xptiInterfaceInfo::Release(void)
         // then we want to bail out of here without destorying the object.
         if(mRefCnt)
             return 1;
-        
+
         if(mEntry)
         {
             mEntry->LockedInterfaceInfoDeathNotification();
